#include "OCIw.h"

void COCIw::DisplayRecords()
{
	size_t rows = GetRecordCount();
	size_t columns = GetFieldCount();

	size_t i, j;

	for(i=0; i<columns; i++) {
		printf("%s", flds_[i].name);
		if(i<columns-1) printf(" | ");
	}
	printf("\n");

	for(i=0; i<rows; i++) {
		for(j=0; j<columns; j++) {
			printf("%s", rset_[i][j]->c_str());
			if(j<columns-1) printf(" | ");
		}
		printf("\n");
	}

	printf("\nTotal Record Count: %u\n", rows);
}

size_t COCIw::GetRecordCount()
{
	return rset_.size();
}

size_t COCIw::GetFieldCount()
{
	return flds_.size();
}

void COCIw::GetRecordData(size_t row, size_t column, char* buf)
{
	strcpy(buf, rset_[row][column]->c_str());
}

void COCIw::clearSet()
{
	flds_.clear();
	
	size_t rcnt = rset_.size();
	if(rcnt==0) return;

	size_t fcnt = rset_[0].size();
	size_t i, j;

	for(i=0; i<rcnt; i++)
		for(j=0; j<fcnt; j++)
			delete rset_[i][j];
}

bool COCIw::Query(const char* SQL)
{
	status = oparse(&cda, (text *)SQL, (sb4)-1, 1, (ub4) 2);
	if (status != 0) 
	{
		cout << "stmt parsing failed." << endl;
		return false;
	}

	sword col, deflen, deftyp;
	static ub1 *defptr;

	struct _cs_datafmt desc[MAX_SELECT_LIST_SIZE];
	struct _column_data def[MAX_SELECT_LIST_SIZE];

	clearSet();

	/* Describe the select-list items. */
	for (col = 0; col < MAX_SELECT_LIST_SIZE; col++)
	{
		desc[col].namelen = MAX_FIELD_NAME_LENGTH;
		if (odescr(&cda, col + 1, &desc[col].maxlength,
			&desc[col].datatype, &desc[col].name[0],
			&desc[col].namelen, &desc[col].dsize,
			&desc[col].precision, &desc[col].scale,
			&desc[col].status))
		{
			/* Break on end of select list. */
			if (cda.rc == VAR_NOT_IN_LIST)
				break;
			else
			{
				//oci_error(cda);
				cout << "get field description error." << endl;
				return false;
			}
		}
		desc[col].name[desc[col].namelen]=0;

		flds_.push_back(desc[col]);

		deflen = desc[col].maxlength > MAX_ITEM_BUFFER_SIZE ? MAX_ITEM_BUFFER_SIZE : desc[col].maxlength+1;
		def[col].value=(ub1*)malloc(deflen);
		defptr = def[col].value;
		deftyp = STRING_TYPE;

		if(odefin(&cda, col + 1,
			defptr, deflen, deftyp,
			-1, &def[col].valuelen, (text *) 0, -1, -1,
			&def[col].col_retlen,
			&def[col].col_retcode))
		{
			cout << "get field data linking error" << endl;
			return false;
		}
	}

	sword col_count = col;

	status = oexec(&cda);
	if (status != 0)
	{
		if(cda.rc!=NO_DATA_FOUND) {
			cout << "stmt execute failed." << endl;
			return false;
		} else {
			cout << "no record found." << endl; // when this code run?
		}
	} else {
		sword col;
		sword ncols = col_count;

		vector<string *> pC;
		for (;;)
		{
			/* Fetch a row.  Break on end of fetch,	disregard null fetch "error". */
			if (ofetch(&cda))
			{
				if (cda.rc == NO_DATA_FOUND) {
//					cout << "query end." << endl;
					break;
				}

				if (cda.rc != NULL_VALUE_RETURNED) {
					// display_error(stderr);
					cout << "get record data error" << endl;
					return false;
				}
			}

			pC.clear();
			for (col=0; col < ncols ; col++)
			{
//				printf("%s|", (char*)(def[col].value));
				string *pV = new string((char*)(def[col].value));
				pC.push_back(pV);
			}
			rset_.push_back(pC);

//			cout << endl;
		}  /* end for (;;) */

//		printf("ڵ尳 : %ld\n", cda.rpc);
	}

	for (col=0; col < col_count ; col++) {
		free(def[col].value);
	}
	//.......................................

	return true;
}

bool COCIw::ExecuteDML(const char* SQL)
{
	//==================================================================================================
	// INSERT, UPDATE, DELETE SQL
	//--------------------------------------------------------------------------------------------------	

	if(!connected) {
		cout << "not DB opened";

		return false;
	}

	status = oparse(&cda, (text *)SQL, (sb4)-1, 1, (ub4) 2);
	if (status != 0) 
	{
		cout << "stmt parsing failed." << endl;
		return false;		
	}

	status = oexec(&cda);
	if (status != 0)
	{
		cout << "stmt execute failed." << endl;
		return false;		
	}

	return true;
	//==================================================================================================
}

COCIw::COCIw()
{
	connected = false;
}

COCIw::~COCIw()
{
	clearSet();
	
	if(connected) CloseDB();
}

bool COCIw::OpenDB(const char* service, const char* user, const char *pw)
{
	connected = true;

	memset(hda,'\0', HDA_SIZE);

	//==================================================================================================
	// oci connect.	
	//--------------------------------------------------------------------------------------------------
	status = olog(&lda, hda, (text *) user, -1, (text *)pw, -1, (text *) service, -1, OCI_LM_DEF);
	if (status != 0) {
		cout << "connection failed." << endl;
		connected = false;
		return false;
	}
	//==================================================================================================

	//==================================================================================================
	// cursor open
	//--------------------------------------------------------------------------------------------------
	status = oopen(&cda, &lda, (text *)0, -1, -1, (text *)0, -1);
	if (status != 0)
	{
		cout << "cursor open failed." << endl;
		connected = false;
		return false;		
	}
	//==================================================================================================		

	return true;
}

bool COCIw::CloseDB()
{
	if(!connected) return false;

	//==================================================================================================
	// cursor close
	//--------------------------------------------------------------------------------------------------
	status = oclose(&cda);
	if (status != 0)
	{
		cout << "cursor close failed." << endl;
		return false;		
	}
	//==================================================================================================

	//==================================================================================================
	// oci disconnect
	//--------------------------------------------------------------------------------------------------
	status = ologof(&lda);
	if (status != 0) {
		cout << "disconnection failed." << endl;
		return false;
	}
	//==================================================================================================

	connected = false;	
	return true;
}